@@ -12,12 +12,12 @@
  * * WebAssembly
  */
 
-use std::boxed::Box;
 use std::collections::HashMap;
 #[cfg(target_os = "macos")]
 use std::ffi::CStr;
 use std::fmt;
-use std::sync::Mutex;
+use std::sync::{Arc, Mutex};
+use std::{boxed::Box, sync::RwLock};
 
 #[cfg(any(target_os = "macos", target_os = "ios"))]
 use cocoa_foundation::base::id;
@@ -157,7 +157,7 @@ unsafe impl Send for UtteranceId {}
 
 unsafe impl Sync for UtteranceId {}
 
-#[derive(Clone, Copy, Debug, Eq, Hash, PartialEq, PartialOrd, Ord)]
+#[derive(Clone, Copy, Debug, Default, Eq, Hash, PartialEq, PartialOrd, Ord)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct Features {
     pub is_speaking: bool,
@@ -168,19 +168,6 @@ pub struct Features {
     pub volume: bool,
 }
 
-impl Default for Features {
-    fn default() -> Self {
-        Self {
-            stop: false,
-            rate: false,
-            pitch: false,
-            volume: false,
-            is_speaking: false,
-            utterance_callbacks: false,
-        }
-    }
-}
-
 impl fmt::Display for Features {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> Result<(), fmt::Error> {
         writeln!(f, "{:#?}", self)
@@ -262,7 +249,7 @@ lazy_static! {
 }
 
 #[derive(Clone)]
-pub struct Tts(Box<dyn Backend>);
+pub struct Tts(Arc<RwLock<Box<dyn Backend>>>);
 
 unsafe impl Send for Tts {}
 
@@ -277,18 +264,18 @@ impl Tts {
             #[cfg(target_os = "linux")]
             Backends::SpeechDispatcher => {
                 let tts = backends::SpeechDispatcher::new()?;
-                Ok(Tts(Box::new(tts)))
+                Ok(Tts(Arc::new(RwLock::new(Box::new(tts)))))
             }
             #[cfg(target_arch = "wasm32")]
             Backends::Web => {
                 let tts = backends::Web::new()?;
-                Ok(Tts(Box::new(tts)))
+                Ok(Tts(Arc::new(RwLock::new(Box::new(tts)))))
             }
             #[cfg(all(windows, feature = "tolk"))]
             Backends::Tolk => {
                 let tts = backends::Tolk::new();
                 if let Some(tts) = tts {
-                    Ok(Tts(Box::new(tts)))
+                    Ok(Tts(Arc::new(RwLock::new(Box::new(tts)))))
                 } else {
                     Err(Error::NoneError)
                 }
@@ -296,20 +283,24 @@ impl Tts {
             #[cfg(windows)]
             Backends::WinRt => {
                 let tts = backends::WinRt::new()?;
-                Ok(Tts(Box::new(tts)))
+                Ok(Tts(Arc::new(RwLock::new(Box::new(tts)))))
             }
             #[cfg(target_os = "macos")]
-            Backends::AppKit => Ok(Tts(Box::new(backends::AppKit::new()))),
+            Backends::AppKit => Ok(Tts(Arc::new(RwLock::new(
+                Box::new(backends::AppKit::new()),
+            )))),
             #[cfg(any(target_os = "macos", target_os = "ios"))]
-            Backends::AvFoundation => Ok(Tts(Box::new(backends::AvFoundation::new()))),
+            Backends::AvFoundation => Ok(Tts(Arc::new(RwLock::new(Box::new(
+                backends::AvFoundation::new(),
+            ))))),
             #[cfg(target_os = "android")]
             Backends::Android => {
                 let tts = backends::Android::new()?;
-                Ok(Tts(Box::new(tts)))
+                Ok(Tts(Arc::new(RwLock::new(Box::new(tts)))))
             }
         };
         if let Ok(backend) = backend {
-            if let Some(id) = backend.0.id() {
+            if let Some(id) = backend.0.read().unwrap().id() {
                 let mut callbacks = CALLBACKS.lock().unwrap();
                 callbacks.insert(id, Callbacks::default());
             }
@@ -362,7 +353,7 @@ impl Tts {
      * Returns the features supported by this TTS engine
      */
     pub fn supported_features(&self) -> Features {
-        self.0.supported_features()
+        self.0.read().unwrap().supported_features()
     }
 
     /**
@@ -373,7 +364,10 @@ impl Tts {
         text: S,
         interrupt: bool,
     ) -> Result<Option<UtteranceId>, Error> {
-        self.0.speak(text.into().as_str(), interrupt)
+        self.0
+            .write()
+            .unwrap()
+            .speak(text.into().as_str(), interrupt)
     }
 
     /**
@@ -382,7 +376,7 @@ impl Tts {
     pub fn stop(&mut self) -> Result<&Self, Error> {
         let Features { stop, .. } = self.supported_features();
         if stop {
-            self.0.stop()?;
+            self.0.write().unwrap().stop()?;
             Ok(self)
         } else {
             Err(Error::UnsupportedFeature)
@@ -393,21 +387,21 @@ impl Tts {
      * Returns the minimum rate for this speech synthesizer.
      */
     pub fn min_rate(&self) -> f32 {
-        self.0.min_rate()
+        self.0.read().unwrap().min_rate()
     }
 
     /**
      * Returns the maximum rate for this speech synthesizer.
      */
     pub fn max_rate(&self) -> f32 {
-        self.0.max_rate()
+        self.0.read().unwrap().max_rate()
     }
 
     /**
      * Returns the normal rate for this speech synthesizer.
      */
     pub fn normal_rate(&self) -> f32 {
-        self.0.normal_rate()
+        self.0.read().unwrap().normal_rate()
     }
 
     /**
@@ -416,7 +410,7 @@ impl Tts {
     pub fn get_rate(&self) -> Result<f32, Error> {
         let Features { rate, .. } = self.supported_features();
         if rate {
-            self.0.get_rate()
+            self.0.read().unwrap().get_rate()
         } else {
             Err(Error::UnsupportedFeature)
         }
@@ -430,10 +424,11 @@ impl Tts {
             rate: rate_feature, ..
         } = self.supported_features();
         if rate_feature {
-            if rate < self.0.min_rate() || rate > self.0.max_rate() {
+            let mut backend = self.0.write().unwrap();
+            if rate < backend.min_rate() || rate > backend.max_rate() {
                 Err(Error::OutOfRange)
             } else {
-                self.0.set_rate(rate)?;
+                backend.set_rate(rate)?;
                 Ok(self)
             }
         } else {
@@ -445,21 +440,21 @@ impl Tts {
      * Returns the minimum pitch for this speech synthesizer.
      */
     pub fn min_pitch(&self) -> f32 {
-        self.0.min_pitch()
+        self.0.read().unwrap().min_pitch()
     }
 
     /**
      * Returns the maximum pitch for this speech synthesizer.
      */
     pub fn max_pitch(&self) -> f32 {
-        self.0.max_pitch()
+        self.0.read().unwrap().max_pitch()
     }
 
     /**
      * Returns the normal pitch for this speech synthesizer.
      */
     pub fn normal_pitch(&self) -> f32 {
-        self.0.normal_pitch()
+        self.0.read().unwrap().normal_pitch()
     }
 
     /**
@@ -468,7 +463,7 @@ impl Tts {
     pub fn get_pitch(&self) -> Result<f32, Error> {
         let Features { pitch, .. } = self.supported_features();
         if pitch {
-            self.0.get_pitch()
+            self.0.read().unwrap().get_pitch()
         } else {
             Err(Error::UnsupportedFeature)
         }
@@ -483,10 +478,11 @@ impl Tts {
             ..
         } = self.supported_features();
         if pitch_feature {
-            if pitch < self.0.min_pitch() || pitch > self.0.max_pitch() {
+            let mut backend = self.0.write().unwrap();
+            if pitch < backend.min_pitch() || pitch > backend.max_pitch() {
                 Err(Error::OutOfRange)
             } else {
-                self.0.set_pitch(pitch)?;
+                backend.set_pitch(pitch)?;
                 Ok(self)
             }
         } else {
@@ -498,21 +494,21 @@ impl Tts {
      * Returns the minimum volume for this speech synthesizer.
      */
     pub fn min_volume(&self) -> f32 {
-        self.0.min_volume()
+        self.0.read().unwrap().min_volume()
     }
 
     /**
      * Returns the maximum volume for this speech synthesizer.
      */
     pub fn max_volume(&self) -> f32 {
-        self.0.max_volume()
+        self.0.read().unwrap().max_volume()
     }
 
     /**
      * Returns the normal volume for this speech synthesizer.
      */
     pub fn normal_volume(&self) -> f32 {
-        self.0.normal_volume()
+        self.0.read().unwrap().normal_volume()
     }
 
     /**
@@ -521,7 +517,7 @@ impl Tts {
     pub fn get_volume(&self) -> Result<f32, Error> {
         let Features { volume, .. } = self.supported_features();
         if volume {
-            self.0.get_volume()
+            self.0.read().unwrap().get_volume()
         } else {
             Err(Error::UnsupportedFeature)
         }
@@ -536,10 +532,11 @@ impl Tts {
             ..
         } = self.supported_features();
         if volume_feature {
-            if volume < self.0.min_volume() || volume > self.0.max_volume() {
+            let mut backend = self.0.write().unwrap();
+            if volume < backend.min_volume() || volume > backend.max_volume() {
                 Err(Error::OutOfRange)
             } else {
-                self.0.set_volume(volume)?;
+                backend.set_volume(volume)?;
                 Ok(self)
             }
         } else {
@@ -553,7 +550,7 @@ impl Tts {
     pub fn is_speaking(&self) -> Result<bool, Error> {
         let Features { is_speaking, .. } = self.supported_features();
         if is_speaking {
-            self.0.is_speaking()
+            self.0.read().unwrap().is_speaking()
         } else {
             Err(Error::UnsupportedFeature)
         }
@@ -572,7 +569,7 @@ impl Tts {
         } = self.supported_features();
         if utterance_callbacks {
             let mut callbacks = CALLBACKS.lock().unwrap();
-            let id = self.0.id().unwrap();
+            let id = self.0.read().unwrap().id().unwrap();
             let mut callbacks = callbacks.get_mut(&id).unwrap();
             callbacks.utterance_begin = callback;
             Ok(())
@@ -594,7 +591,7 @@ impl Tts {
         } = self.supported_features();
         if utterance_callbacks {
             let mut callbacks = CALLBACKS.lock().unwrap();
-            let id = self.0.id().unwrap();
+            let id = self.0.read().unwrap().id().unwrap();
             let mut callbacks = callbacks.get_mut(&id).unwrap();
             callbacks.utterance_end = callback;
             Ok(())
@@ -616,7 +613,7 @@ impl Tts {
         } = self.supported_features();
         if utterance_callbacks {
             let mut callbacks = CALLBACKS.lock().unwrap();
-            let id = self.0.id().unwrap();
+            let id = self.0.read().unwrap().id().unwrap();
             let mut callbacks = callbacks.get_mut(&id).unwrap();
             callbacks.utterance_stop = callback;
             Ok(())
@@ -646,9 +643,11 @@ impl Tts {
 
 impl Drop for Tts {
     fn drop(&mut self) {
-        if let Some(id) = self.0.id() {
-            let mut callbacks = CALLBACKS.lock().unwrap();
-            callbacks.remove(&id);
+        if Arc::strong_count(&self.0) <= 1 {
+            if let Some(id) = self.0.read().unwrap().id() {
+                let mut callbacks = CALLBACKS.lock().unwrap();
+                callbacks.remove(&id);
+            }
         }
     }
 }
